/**
 * Copyright (c) 2013-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.registry_overlay.api;

import java.util.Arrays;
import java.util.Collection;
import java.util.Collections;
import java.util.Map;

import org.ow2.petals.topology.generated.Topology;

import com.hazelcast.core.HazelcastInstance;
import com.hazelcast.core.IMap;
import com.hazelcast.nio.serialization.PortableWriter;

/**
 * The data model of the registry overlay as distributed map, distributed multi map, distributed lock, ...
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class DataModelApi {

    /**
     * Pattern of the map name containing the topologies.
     */
    public static final String TOPOLOGIES_MAP_NAME_PATTERN = "Topologies";

    /**
     * Pattern of the map name containing the endpoints
     */
    public static final String ENDPOINT_MAP_NAME_PATTERN = "%s: Endpoints";

    /**
     * Pattern of the map name containing the endpoint descriptions
     */
    public static final String ENDPOINT_DESCRIPTION_MAP_NAME_PATTERN = "%s: Endpoint descriptions";

    static final String ENDPOINT_TYPE_FIELD = "type";

    static final String ENDPOINT_NAME_FIELD = "endpoint";

    static final String ENDPOINT_SERVICE_FIELD = "service";

    static final String ENDPOINT_INTERFACES_FIELD = "interfaces";

    static final String ENDPOINT_CONTAINER_FIELD = "container";

    static final String ENDPOINT_COMPONENT_FIELD = "component";

    static final String ENDPOINT_RESTORATION_FIELD = "restoration";

    public static final String ENDPOINT_INTERFACES_INDEX = ENDPOINT_INTERFACES_FIELD + "[any]";

    public static final String ENDPOINT_RESTAURATION_INDEX = ENDPOINT_RESTORATION_FIELD;

    public static final String ENDPOINT_SERVICE_INDEX = ENDPOINT_SERVICE_FIELD;

    /**
     * Note: it is stored as a {@link String} and not an {@link Enum}, so for comparisons, {@link Enum#name()} should be
     * used!
     * 
     * TODO simplify it when {@link PortableWriter} will support enums
     */
    public static final String ENDPOINT_TYPE_INDEX = ENDPOINT_TYPE_FIELD;

    public static final String ENDPOINT_CONTAINER_INDEX = ENDPOINT_CONTAINER_FIELD;

    public static final Collection<String> INDEXES = Collections
            .unmodifiableList(Arrays.asList(ENDPOINT_INTERFACES_INDEX, ENDPOINT_RESTAURATION_INDEX,
                    ENDPOINT_SERVICE_INDEX, ENDPOINT_TYPE_INDEX, ENDPOINT_CONTAINER_INDEX));

    private DataModelApi() {
        // Utility class, no constructor
    }

    /**
     * Get the name of distributed map containing topologies.
     * 
     * @return The distributed map name
     */
    public static String getTopologiesMapName() {
        return String.format(TOPOLOGIES_MAP_NAME_PATTERN);
    }

    /**
     * Get the name of distributed map containing endpoints.
     * 
     * @param registryName
     *            Registry name of the current container
     * @return The distributed map name
     */
    public static String getEndpointsMapName(final String registryName) {
        return String.format(ENDPOINT_MAP_NAME_PATTERN, registryName);
    }

    /**
     * Get the name of distributed map containing endpoint descriptions.
     * 
     * @param registryName
     *            Registry name of the current container
     * @return The distributed map name
     */
    public static String getEndpointDescriptionsMapName(final String registryName) {
        return String.format(ENDPOINT_DESCRIPTION_MAP_NAME_PATTERN, registryName);
    }

    /**
     * <p>
     * Get the distributed map containing topologies.
     * </p>
     * <p>
     * <b>Caution </b> : Access to elements of this map MUST be protected using one of the lock method of {@link IMap}.
     * </p>
     * 
     * @param hzlInst
     *            The current HZL instance
     * @return The distributed map
     */
    public static IMap<String, Topology> getTopologiesMap(final HazelcastInstance hzlInst) {
        return hzlInst.getMap(DataModelApi.getTopologiesMapName());
    }

    /**
     * Get the distributed map containing endpoints
     * 
     * @param hzlInst
     *            The current HZL instance
     * @param registryName
     *            Registry name of the current container
     * @return The distributed map
     */
    public static IMap<String, StoredRegistryEndpoint> getEndpointsMap(final HazelcastInstance hzlInst,
            final String registryName) {
        return hzlInst.getMap(DataModelApi.getEndpointsMapName(registryName));
    }

    /**
     * Get the distributed map containing endpoint descriptions
     * 
     * @param hzlInst
     *            The current HZL instance
     * @param registryName
     *            Registry name of the current container
     * @return The distributed map
     */
    public static IMap<String, String> getEndpointDescriptionsMap(final HazelcastInstance hzlInst,
            final String registryName) {
        return hzlInst.getMap(DataModelApi.getEndpointDescriptionsMapName(registryName));
    }

    /**
     * Clean all information of a given topology
     * 
     * @param hzlInst
     *            The current HZL instance
     * @param registryName
     *            Registry name associated to the topology to clean
     * @throws UnexistingTopologyException
     *             The given topology does not exist
     * @return <code>true</code> if it succeeded or <code>false</code> if the topology didn't exist
     */
    public static boolean cleanTopology(final HazelcastInstance hzlInst, final String registryName) {
        final Map<String, Topology> topologyMap = DataModelApi.getTopologiesMap(hzlInst);
        if (topologyMap.containsKey(registryName)) {
            DataModelApi.getEndpointsMap(hzlInst, registryName).clear();
            DataModelApi.getEndpointDescriptionsMap(hzlInst, registryName).clear();
            topologyMap.remove(registryName);
            return true;
        } else {
            return false;
        }
    }

}
